import sys
import argparse
# needs pip install Pillow
from PIL import Image

#####################
# setup/config
#####################
parser = argparse.ArgumentParser()
parser.add_argument('--img', help = 'The sprite sheet image filename.', default = "", type = file, required = True)
parser.add_argument('--alias', help = 'A name to associate these frames with when using the spritesheet.', default = "", type = str, required = True)
parser.add_argument('--ignoreTransparentFrames', help = 'If True, the generated .plist will ignore empty (transparent) frames.', default = True, type = bool, required = False)
parser.add_argument('--frameSize', help = 'Width of each animation frame (width height).', required = True, nargs = 2)
parser.add_argument('--plist', help = 'Preferred name of the generated .plist file. If omitted, the generated filename will be the same as the image filename.', default = "", type = str, required = False)
parser.add_argument('--zeroIndexed', help = 'Whether the frame indexes should be zero indexed or not.', default = True, type = bool, required = False)
args = parser.parse_args()

print ""

outputName = ""
if args.plist == "":
    outputName = args.img.name.replace(".png", ".plist")
    print "INFO: No output name was provided. Output name will be '{outputName}'".format(outputName = outputName)

img = Image.open(args.img.name);
frameWidth = int(args.frameSize[0])
frameHeight = int(args.frameSize[1])
srcImgWidth = img.size[0]
srcImgHeight = img.size[1]

if srcImgWidth % frameWidth != 0:
    print "ERROR: Source image width needs to be trimmed to match frame width"
    sys.exit(0)

if srcImgHeight % frameHeight != 0:
    print "ERROR: Source image height needs to be trimmed to match frame height"
    sys.exit(0)

#####################
# header
#####################
plistHeader = """<?xml version="1.0" encoding="UTF-8"?>\n<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">\n<plist version="1.0">\n"""
contentStart = """  <dict>\n    <key>frames</key>\n    <dict>"""
framesBlock = ""
lastX = 0
lastY = 0
numFramesAdded = 0
numFramesSkipBecauseTransparent = 0
skippedFrames = []

#####################
# add frame entries
#####################

# make it look like:
# {{1,1},{40,38}}
while True:

    if args.ignoreTransparentFrames:
        sample = img.crop((lastX, lastY, lastX + frameWidth, lastY + frameHeight))
        colors = sample.getcolors(100000) # max colors

        # if completely transparent, list will only contain one color
        if len(colors) > 0 and len(colors) < 100:
            # color format
            # [(color count, (r, g, b, a))]
            # [(4900, (255, 255, 255, 0))]
            if colors[0][1][3] == 0:
                numFramesSkipBecauseTransparent += 1
                skippedFrames.append((lastX, lastY))

                lastX += frameWidth
                if lastX >= srcImgWidth:
                    lastX = 0
                    lastY += frameHeight

                    if lastY >= srcImgHeight:
                        break
                else:
                    continue

    nextSrcRec = "{{" + str(lastX) + "," + str(lastY) + "}" + "," + "{" + str(frameWidth) + ","  + str(frameHeight) + "}}"
    framesBlock += """\n      <key>{frameAlias}</key>\n      <dict>\n        <key>aliases</key>\n        <array/>\n        <key>spriteOffset</key>\n        <string>{offset}</string>\n        <key>spriteSize</key>\n        <string>{frameWidth},{frameHeight}</string>\n        <key>spriteSourceSize</key>\n        <string>{spriteWidth},{spriteHeight}</string>\n        <key>textureRect</key>\n        <string>{srcRec}</string>\n        <key>textureRotated</key>\n        <false/>\n      </dict>""".format(
                                        frameAlias = args.alias + (str(numFramesAdded) if args.zeroIndexed else str(numFramesAdded + 1)),
                                        frameWidth = '{' + str(frameWidth),
                                        frameHeight = str(frameHeight) + "}",
                                        offset = "{0,0}",
                                        spriteWidth = '{' + str(frameWidth),
                                        spriteHeight = str(frameHeight) + "}",
                                        srcRec = nextSrcRec)

    numFramesAdded += 1
    lastX += frameWidth
    if lastX >= srcImgWidth:
        lastX = 0
        lastY += frameHeight

        if lastY >= srcImgHeight:
            break

#####################
# adds metadata block
#####################
metaStart = "\n    <key>metadata</key>\n    <dict>"
metaFormat = """\n      <key>format</key>\n      <integer>{version}</integer>""".format(version = 3)
metaPxFormat = """\n      <key>pixelFormat</key>\n      <string>{format}</string>""".format(format = 'RGBA8888')
metaPreMultiplyAlpha = """\n      <key>premultiplyAlpha</key>\n      <false/>"""
metaRealTextureFilename = """\n      <key>realTextureFileName</key>\n      <string>{png}</string>""".format(png = args.img.name)
metaSize = """\n      <key>size</key>\n      <string>{srcImgWidth},{srcImgHeight}</string>""".format(srcImgWidth = "{" + str(srcImgWidth), srcImgHeight = str(srcImgHeight) + "}")
metaTextureFileName = """\n      <key>textureFileName</key>\n      <string>{png}</string>""".format(png = args.img.name)

######################
# put it all together
######################
plistFooter = """\n</plist>"""
metaFooter = """\n    </dict>"""
plistMetadata = metaStart + metaFormat + metaPxFormat + metaPreMultiplyAlpha + metaRealTextureFilename + metaSize + metaTextureFileName + metaFooter + "\n  </dict>" + plistFooter

print ""
print "....................................................................."
print plistHeader
print contentStart
print framesBlock
print plistMetadata
print "....................................................................."
print "Frame(s) added: {0}".format(str(numFramesAdded))
print "Frames skipped b/c transparent: {0}; {1}".format(str(numFramesSkipBecauseTransparent), skippedFrames)
print "Frame with was: " + str(frameWidth)
print "Frame height was: " + str(frameHeight)
print "Spritesheet width was: " + str(srcImgWidth)
print "Spritesheet height was: " + str(srcImgHeight)
print ""

plistContent = plistHeader + contentStart + framesBlock + "\n    </dict>" + plistMetadata
plist = open(outputName, "w")
plist.write(plistContent)
plist.close()
